import pygame
from pygame.locals import *

import pathfind
import random

import numpy as np
from collections import deque

from avatar import *

level = pathfind.level1

# Small tweak to the game loop when running in IDLE to avoid
# killing the IDLE window.
running = True

pygame.init()

screen = pygame.display.set_mode((640,480), 0, 32)

# To easily construct bounding boxes, images are now going to be rectangles.
boy_image = pygame.Surface((25,25))
boy_image.fill((135,145,191))
                    
girl_image = pygame.Surface((25,25))
girl_image.fill((255,192,203))

boy = Avatar("Boy", boy_image, (25.0,25.0), 200.0)
girl = Avatar("Girl", girl_image, (300.0, 300.0), 50.0)

clock = pygame.time.Clock()

health_image = pygame.Surface((10,10))
health_image.fill((0,255,0))

ammo_image = pygame.Surface((10,10))
ammo_image.fill((255,0,0))

items = {
  'health': None,
  'ammo': None
}

# Load an array of tiles to translate the level to a block.
tiles = {
  ".": (50,50,50),
  "#": (128,0,0)  
}

# Knowledge is stored in the blackboard.

# Scratch location for data.
enemy_bb = {
  'health' : 100,
  'ammo' : 10,
  'time_passed' : 0,
  'delay': 0,
  'action': 'Idle',
  'panic': None
}

player_bb = {
  'health' : 100,
  'ammo' : 20
}

def grabItem(avatar, blackboard):

  if avatarPositionToTile(avatar) == items['health']:

    items['health'] = None
    blackboard['health'] = blackboard['health'] + 10
    
    if blackboard['health'] > 100:
      blackboard['health'] = 100

  if avatarPositionToTile(avatar) == items['ammo']:

    items['ammo'] = None
    blackboard['ammo'] = blackboard['ammo'] + 10
    
    if blackboard['ammo'] > 100:
      blackboard['ammo'] = 100
  
def randomEvent(level):

  rows = len(level)
  cols = len(level[0])

  if items['health'] is None:

    while True:
      x = random.randint(0, cols - 1)
      y = random.randint(0, rows - 1)

      if level[y][x] == '.' and items['ammo'] != (x, y):
        break

    items['health'] = (x, y)

  if items['ammo'] is None:

    while True:
      x = random.randint(0, cols - 1)
      y = random.randint(0, rows - 1)

      if level[y][x] == '.' and items['health'] != (x, y):
        break

    items['ammo'] = (x, y)

# The height and width of each tile.
TILE_WIDTH = 100
TILE_HEIGHT = 100

# Low-level seek movement; ai should be an avatar object.
def seek(ai, to, time_passed):
  target = to - ai.position
  ai.update(target, time_passed)

# Player has noclip. This function takes a tile coordinate
# and converts it to pixels so that the pixels are in
# the center of the tile.
def tilePositionToPixel(tile):

  return (np.array((TILE_WIDTH / 2, TILE_HEIGHT / 2)) +
          np.array((TILE_WIDTH, TILE_HEIGHT)) * tile)

def avatarPositionToTile(avatar):
  x, y = avatar.position
  return (int(x) / TILE_WIDTH, int(y) / TILE_HEIGHT)

def findNext(level, ai, target):

    # Figure out which tile you and your enemy are by
    # unpacking the player.    
    ai_x, ai_y = ai.position
    target_x, target_y = target

    player_tile = (int(target_x) / TILE_WIDTH, int(target_y) / TILE_HEIGHT)
    enemy_tile = (int(ai_x)/ TILE_WIDTH, int(ai_y) / TILE_HEIGHT)

    path = pathfind.bfs(level, enemy_tile, player_tile)

    if path == None:
      return None
    if len(path) == 1:
      return tilePositionToPixel(path[0])
    else:
      return tilePositionToPixel(path[1])      
  
def drawBoard(level, tiles, screen):
  
  width = len(level) * TILE_WIDTH
  height = len(level[0]) * TILE_HEIGHT
 
  for x in range(len(level[0])):
    for y in range(len(level)):

      where = (x * TILE_WIDTH, y * TILE_HEIGHT, TILE_WIDTH, TILE_HEIGHT)
      color = tiles[level[y][x]]

      if level[y][x] == '#':
        pygame.draw.rect(screen, color, Rect(where))
      else:
        pygame.draw.rect(screen, color, Rect(where), 1)
            
def aiDecisionTree(level, ai, player, ai_bb, player_bb):

  # Health action
  if (ai_bb['health'] < 20 and items['health'] is not None):
    enemy_bb['action'] = 'Health'
    find_next = findNext(level, ai, tilePositionToPixel(items['health']))
    seek(ai, find_next, enemy_bb['time_passed'])
    return
        
  # Fire action
  elif (ai_bb['ammo'] > 0 and
      avatarPositionToTile(player) == avatarPositionToTile(ai)):

      enemy_bb['action'] = 'Fire'

      if ai_bb['delay'] > 1:
        ai_bb['delay'] = 0
        ai_bb['ammo'] = ai_bb['ammo'] - 1
        player_bb['health'] = player_bb['health'] - 1
      
      return
  
  # Find ammo action
  elif (ai_bb['ammo'] == 0 and items['ammo'] is not None):
    enemy_bb['action'] = 'Ammo'
    find_next = findNext(level, ai, tilePositionToPixel(items['ammo']))    
    seek(ai, find_next, enemy_bb['time_passed'])
    return
  
  # Seek
  else:
    enemy_bb['action'] = 'Seek'
    find_next = findNext(level, ai, player.position)    
    if find_next is not None:
      seek(ai, find_next, enemy_bb['time_passed'])

def statusline(screen):

  # Fancy font handling.
  status = 'P Health {} Ammo {}'.format(
  player_bb['health'], player_bb['ammo'])

  font = pygame.font.SysFont("Consolas", 20)
  text = font.render(status, True, (255,255,255), (0,0,0))

  textRect = text.get_rect()
  textRect.topleft = (10,450)

  screen.blit(text, textRect)

  status = 'E Health {} Ammo {} | {}'.format(    
    enemy_bb['health'], enemy_bb['ammo'], enemy_bb['action'])

  text = font.render(status, True, (255,255,255), (0,0,0))
  textRect = text.get_rect()
  textRect.topright = (630, 450)

  screen.blit(text, textRect)
      
while running:

  time_passed = clock.tick(30)
  time_passed_seconds = time_passed / 1000.0

  # Slow down the AI  
  enemy_bb['time_passed'] = time_passed_seconds
  enemy_bb['delay'] = enemy_bb['delay'] + time_passed_seconds

  for event in pygame.event.get():
    
    if (event.type == QUIT or
        (event.type == KEYDOWN and event.key == K_ESCAPE)):

      running = False
   
    if (event.type == KEYDOWN):

      if event.key == K_r:
        randomEvent(level)
        
      if event.key == K_SPACE:

        if player_bb['ammo'] > 0:
          player_bb['ammo'] = player_bb['ammo'] - 1

          if avatarPositionToTile(boy) == avatarPositionToTile(girl):
            enemy_bb['health'] = enemy_bb['health'] - 5
      
  pressed_keys = pygame.key.get_pressed()

  target = np.array([0.0,0.0])

  # Handle keyboard presses.
  if pressed_keys[K_LEFT]:
    target += np.array([-1.0,0.0])
  if pressed_keys[K_RIGHT]:
    target += np.array([1.0,0.0])
  if pressed_keys[K_UP]:
    target += np.array([0.0,-1.0])
  if pressed_keys[K_DOWN]:
    target += np.array([0.0,1.0])

  boy.update(target, time_passed_seconds)

  screen.fill((0,0,0))
    
  drawBoard(level, tiles, screen)
  statusline(screen)
  
  # Needs to be repositioned.
  screen.blit(girl.surface, girl.position - girl.surface.get_rect().center)
  screen.blit(boy.surface, boy.position - girl.surface.get_rect().center)  

  grabItem(boy, player_bb)
  grabItem(girl, enemy_bb)

  # Draw the special items.
  if items['health'] is not None:
    screen.blit(health_image, tilePositionToPixel(items['health'])
                - health_image.get_rect().center)

  if items['ammo'] is not None:    
    screen.blit(ammo_image, tilePositionToPixel(items['ammo'])
                - ammo_image.get_rect().center)

  # Change this to AI behavior.
  aiDecisionTree(level, girl, boy, enemy_bb, player_bb)
    
  pygame.display.update()

# Wrap up.
pygame.quit()
