import pygame
from pygame.locals import *

import random
import pathfind

import numpy as np
from collections import deque

from avatar import *

# Small tweak to the game loop when running in IDLE to avoid
# killing the IDLE window.
running = True

pygame.init()

screen = pygame.display.set_mode((640,480), 0, 32)

boy_image = pygame.Surface((25,25))
boy_image.fill((135,145,191))
                    
girl_image = pygame.Surface((25,25))
girl_image.fill((255,192,203))

boy = Avatar("Boy", boy_image, (50.0,50.0), 200.0)
girl = Avatar("Girl", girl_image, (150.0, 50.0), 50.0)

clock = pygame.time.Clock()

tiles = {
  ".": (50,50,50),
  "#": (128,0,0)
}

TILE_WIDTH = 100
TILE_HEIGHT = 100

# Generates a random level.
def randomLevel():
  global level

  rows = random.randint(3, 4)
  cols = random.randint(3, 6)

  level = []
  for r in range(0, rows):
    row = ['.'] * cols
    level.append(row)
    
  for i in range(0, rows * cols / 3):
    r = random.randint(0, rows - 1)
    c = random.randint(0, cols - 1)
    level[r][c] = '#'
      
# Low-level seek movement; ai should be an avatar object.
def seek(ai, to, time_passed):
  target = to - ai.position
  ai.update(target, time_passed)

# Player has noclip. This function takes a tile coordinate
# and converts it to pixels so that the pixels are in
# the center of the tile.
def tilePositionToPixel(tile):

  return (np.array((TILE_WIDTH / 2, TILE_HEIGHT / 2)) +
          np.array((TILE_WIDTH, TILE_HEIGHT)) * tile)

def findNext(level, ai, player):

    # Figure out which tile you and your enemy are by
    # unpacking the player.
    
    ai_x, ai_y = ai.position
    player_x, player_y = player.position

    player_tile = (int(player_x) / TILE_WIDTH, int(player_y) / TILE_HEIGHT)
    enemy_tile = (int(ai_x)/ TILE_WIDTH, int(ai_y) / TILE_HEIGHT)

    path = pathfind.bfs(level, enemy_tile, player_tile)

    if path == None:
      return None
    if len(path) == 1:
      return tilePositionToPixel(path[0])
    else:
      return tilePositionToPixel(path[1])      
  
def drawBoard(level, tiles, screen):
  
  width = len(level) * TILE_WIDTH
  height = len(level[0]) * TILE_HEIGHT
 
  for x in range(len(level[0])):
    for y in range(len(level)):

      where = (x * TILE_WIDTH, y * TILE_HEIGHT, TILE_WIDTH, TILE_HEIGHT)
      color = tiles[level[y][x]]

      if level[y][x] == '#':
        pygame.draw.rect(screen, color, Rect(where))
      else:
        pygame.draw.rect(screen, color, Rect(where), 1)

# Generate a random level.
randomLevel()

while running:

  time_passed = clock.tick(30)
  time_passed_seconds = time_passed / 1000.0

  for event in pygame.event.get():
    
    if (event.type == QUIT or
        (event.type == KEYDOWN and event.key == K_ESCAPE)):

      running = False

      
  pressed_keys = pygame.key.get_pressed()

  target = np.array([0.0,0.0])

  # Handle keyboard presses.
  if pressed_keys[K_LEFT]:
    target += np.array([-1.0,0.0])
  if pressed_keys[K_RIGHT]:
    target += np.array([1.0,0.0])
  if pressed_keys[K_UP]:
    target += np.array([0.0,-1.0])
  if pressed_keys[K_DOWN]:
    target += np.array([0.0,1.0])

  boy.update(target, time_passed_seconds)

  screen.fill((0,0,0))

  drawBoard(level, tiles, screen)
  
  screen.blit(girl.surface, girl.position - girl.surface.get_rect().center)
  screen.blit(boy.surface, boy.position - girl.surface.get_rect().center)  
    
  find_next = findNext(level, girl, boy)

  if find_next is not None:
    seek(girl, find_next, time_passed_seconds)

  pygame.display.update()

# Wrap up.
pygame.quit()
